#include "IIC.h"

#define delay() for(i=IIC_Delay;i>0;i--)
unsigned char i;					//Control variable

void IIC_Start(void)
{
	IIC_Data_DIR |= IIC_SCL_BIT;	//Init SCL Output
	SDA_OUT;						//Init SDA Output

	SCL_H;
    delay();
	SDA_H;
    delay();
	SDA_L;
    delay();
	SCL_L;
    delay();

	return;
}

void IIC_Stop(void){
	SDA_OUT;

	SCL_L;
	SDA_L;
	SCL_H;
	SDA_H;

	return;
}

unsigned char IIC_Write_Byte(unsigned char byte){
	SDA_OUT;

	SCL_L;
    delay();
	for(i = 8;i > 0;i --)
	{
		if(byte & 0x80)
		{
			SDA_H;
            delay();
		}
		else
		{
			SDA_L;
            delay();
		}
		SCL_H;
        delay();
		SCL_L;
        delay();
		byte <<= 1;
	}
	SDA_H;
    delay();
	SDA_IN;
    delay();
	SCL_H;
    delay();
	i = SDA_I;				//Read ACK
	SCL_L;
	return i;
}

unsigned char IIC_Read_MSByte(void){
	unsigned char data;
	data = 0;
	SDA_OUT;

    SDA_H;
    delay();
	SDA_IN;
    delay();
	SCL_L;
    delay();
	for(i = 8;i > 0;i --)
	{
		data <<= 1;
		data |= SDA_I;
        delay();
		SCL_H;
        delay();
		SCL_L;
        delay();
	}

	SDA_OUT;				//Send ACK
    delay();
	SDA_L;
    delay();
	SCL_H;
    delay();
	SCL_L;
    delay();

	return data;
}

void IIC_Write(unsigned char const dev_add,unsigned char const *write_cmd,unsigned char const cmd_size,unsigned char const *data,unsigned char const data_size)
{
	IIC_Start();
    delay();
	if(IIC_Write_Byte((dev_add << 1) & 0xFE))
	{
		return;		//No response from slaves
	}
    delay();
	for(i=0;i<cmd_size;i++)
	{
		IIC_Write_Byte(write_cmd[i]);
        delay();
	}
	for(i=0;i<data_size;i++)
	{
		IIC_Write_Byte(data[i]);
        delay();
	}
	IIC_Stop();
	return;
}

void IIC_Read(unsigned char const dev_add,unsigned char const *read_cmd,unsigned char const cmd_size,unsigned char *data,unsigned char const data_size)
{
	IIC_Start();
    delay();
	if(IIC_Write_Byte((dev_add << 1) & 0xFE))
	{
		return;		//No response from slaves
	}
	for(i=0;i<cmd_size;i++)
	{
		IIC_Write_Byte(read_cmd[i]);
        delay();
	}
	IIC_Start();
	if(IIC_Write_Byte((dev_add << 1) | 0x01))
	{
		return;		//No response from slaves
	}
    delay();
	for(i=0;i<data_size;i++)
	{
		data[i] = IIC_Read_MSByte();
	}
	IIC_Stop();
       
	return;
}
